"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Class = void 0;
const Term_1 = require("./Term");
const reasoning_1 = require("./reasoning");
const namespaces_1 = require("./namespaces");
class Class extends Term_1.Term {
    constructor(IRI, graph) {
        super(IRI, graph);
        this.termTypeLabel = namespaces_1.TermTypeLabel.class;
        this.termTypeIRI = namespaces_1.TermTypeIRI.class;
    }
    getTermObj() {
        return this.graph.classes[this.IRI];
    }
    getProperties(implicit = true, filter) {
        const classObj = this.getTermObj();
        const result = [];
        result.push(...classObj[namespaces_1._SOA.hasProperty]);
        if (implicit) {
            result.push(...(0, reasoning_1.inferPropertiesFromSuperClasses)(classObj[namespaces_1._RDFS.subClassOf], this.graph));
        }
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    getSuperClasses(implicit = true, filter) {
        const classObj = this.getTermObj();
        const result = [];
        if (implicit) {
            result.push(...(0, reasoning_1.inferSuperClasses)(this.IRI, this.graph));
        }
        else {
            result.push(...classObj[namespaces_1._RDFS.subClassOf]);
        }
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    getSubClasses(implicit = true, filter) {
        const classObj = this.getTermObj();
        const result = [];
        if (implicit) {
            result.push(...(0, reasoning_1.inferSubClasses)(this.IRI, this.graph));
        }
        else {
            result.push(...classObj[namespaces_1._SOA.superClassOf]);
        }
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    isRangeOf(implicit = true, filter) {
        const result = [];
        if (implicit) {
            result.push(...(0, reasoning_1.inferRangeOf)(this.IRI, this.graph));
        }
        else {
            result.push(...this.getTermObj()[namespaces_1._SOA.isRangeOf]);
        }
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    toString(implicit = true, filter) {
        return JSON.stringify(this.toJSON(implicit, filter), null, 2);
    }
    toJSON(implicit = true, filter) {
        const result = super.toJSON();
        result.superClasses = this.getSuperClasses(implicit, filter);
        result.subClasses = this.getSubClasses(implicit, filter);
        result.properties = this.getProperties(implicit, filter);
        result.rangeOf = this.isRangeOf(implicit, filter);
        return result;
    }
}
exports.Class = Class;
//# sourceMappingURL=Class.js.map