"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataType = void 0;
const Term_1 = require("./Term");
const reasoning_1 = require("./reasoning");
const namespaces_1 = require("./namespaces");
class DataType extends Term_1.Term {
    constructor(IRI, graph) {
        super(IRI, graph);
        this.termTypeLabel = namespaces_1.TermTypeLabel.dataType;
        this.termTypeIRI = namespaces_1.TermTypeIRI.dataType;
    }
    getTermObj() {
        return this.graph.dataTypes[this.IRI];
    }
    getSuperDataTypes(implicit = true, filter) {
        const dataTypeObj = this.getTermObj();
        const result = [];
        if (implicit) {
            result.push(...(0, reasoning_1.inferSuperDataTypes)(this.IRI, this.graph));
        }
        else {
            result.push(...dataTypeObj[namespaces_1._RDFS.subClassOf]);
        }
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    getSubDataTypes(implicit = true, filter) {
        const dataTypeObj = this.getTermObj();
        const result = [];
        if (implicit) {
            result.push(...(0, reasoning_1.inferSubDataTypes)(this.IRI, this.graph));
        }
        else {
            result.push(...dataTypeObj[namespaces_1._SOA.superClassOf]);
        }
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    isRangeOf(implicit = true, filter) {
        const result = [];
        if (implicit) {
            result.push(...(0, reasoning_1.inferRangeOf)(this.IRI, this.graph));
        }
        else {
            result.push(...this.getTermObj()[namespaces_1._SOA.isRangeOf]);
        }
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    toJSON(implicit = true, filter) {
        const result = super.toJSON();
        result.superDataTypes = this.getSuperDataTypes(implicit, filter);
        result.subDataTypes = this.getSubDataTypes(implicit, filter);
        result.rangeOf = this.isRangeOf(implicit, filter);
        return result;
    }
    toString(implicit = true, filter) {
        return JSON.stringify(this.toJSON(implicit, filter), null, 2);
    }
}
exports.DataType = DataType;
//# sourceMappingURL=DataType.js.map