"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Enumeration = void 0;
const Class_1 = require("./Class");
const utilities_1 = require("./utilities");
const namespaces_1 = require("./namespaces");
const reasoning_1 = require("./reasoning");
class Enumeration extends Class_1.Class {
    constructor(IRI, graph) {
        super(IRI, graph);
        this.termTypeLabel = namespaces_1.TermTypeLabel.enumeration;
        this.termTypeIRI = namespaces_1.TermTypeIRI.enumeration;
    }
    getTermObj() {
        return this.graph.enumerations[this.IRI];
    }
    getEnumerationMembers(implicit = true, filter) {
        const result = [];
        result.push(...this.getTermObj()[namespaces_1._SOA.hasEnumerationMember]);
        if (implicit) {
            const subClasses = this.getSubClasses(true);
            for (const actSubClass of subClasses) {
                const actualEnumeration = this.graph.enumerations[actSubClass];
                if (!(0, utilities_1.isNil)(actualEnumeration)) {
                    result.push(...actualEnumeration[namespaces_1._SOA.hasEnumerationMember]);
                }
            }
        }
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    toString(implicit = true, filter) {
        return JSON.stringify(this.toJSON(implicit, filter), null, 2);
    }
    toJSON(implicit = true, filter) {
        const result = super.toJSON(implicit, filter);
        result.enumerationMembers = this.getEnumerationMembers(implicit, filter);
        return result;
    }
}
exports.Enumeration = Enumeration;
//# sourceMappingURL=Enumeration.js.map