"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnumerationMember = void 0;
const Term_1 = require("./Term");
const utilities_1 = require("./utilities");
const reasoning_1 = require("./reasoning");
const namespaces_1 = require("./namespaces");
class EnumerationMember extends Term_1.Term {
    constructor(IRI, graph) {
        super(IRI, graph);
        this.termTypeLabel = namespaces_1.TermTypeLabel.enumerationMember;
        this.termTypeIRI = namespaces_1.TermTypeIRI.enumerationMember;
    }
    getTermObj() {
        return this.graph.enumerationMembers[this.IRI];
    }
    getDomainEnumerations(implicit = true, filter) {
        const enumObj = this.getTermObj();
        let result = [];
        result.push(...enumObj[namespaces_1._SOA.enumerationDomainIncludes]);
        if (implicit) {
            const domainEnumerationsToCheck = (0, utilities_1.cloneJson)(result);
            for (const actDE of domainEnumerationsToCheck) {
                result.push(...(0, reasoning_1.inferSuperClasses)(actDE, this.graph));
            }
            result = (0, reasoning_1.applyFilter)({
                data: result,
                filter: { termType: namespaces_1.TermTypeLabel.enumeration },
                graph: this.graph,
            });
        }
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    toString(implicit = true, filter) {
        return JSON.stringify(this.toJSON(implicit, filter), null, 2);
    }
    toJSON(implicit = true, filter) {
        const result = super.toJSON();
        result["domainEnumerations"] = this.getDomainEnumerations(implicit, filter);
        return result;
    }
}
exports.EnumerationMember = EnumerationMember;
//# sourceMappingURL=EnumerationMember.js.map