"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Property = void 0;
const Term_1 = require("./Term");
const reasoning_1 = require("./reasoning");
const namespaces_1 = require("./namespaces");
class Property extends Term_1.Term {
    constructor(IRI, graph) {
        super(IRI, graph);
        this.termTypeLabel = namespaces_1.TermTypeLabel.property;
        this.termTypeIRI = namespaces_1.TermTypeIRI.property;
    }
    getTermObj() {
        return this.graph.properties[this.IRI];
    }
    getRanges(implicit = true, filter) {
        const propertyObj = this.getTermObj();
        const result = [];
        result.push(...propertyObj[namespaces_1._SCHEMA.rangeIncludes]);
        if (implicit) {
            for (const actRes of result) {
                result.push(...(0, reasoning_1.inferSubDataTypes)(actRes, this.graph));
            }
            for (const actRes of result) {
                result.push(...(0, reasoning_1.inferSubClasses)(actRes, this.graph));
            }
        }
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    getDomains(implicit = true, filter) {
        const propertyObj = this.getTermObj();
        const result = [];
        result.push(...propertyObj[namespaces_1._SCHEMA.domainIncludes]);
        if (implicit) {
            const inferredSubClasses = [];
            for (const actRes of result) {
                inferredSubClasses.push(...(0, reasoning_1.inferSubClasses)(actRes, this.graph));
            }
            result.push(...inferredSubClasses);
        }
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    getSuperProperties(implicit = true, filter) {
        const propertyObj = this.getTermObj();
        const result = [];
        if (implicit) {
            result.push(...(0, reasoning_1.inferSuperProperties)(this.IRI, this.graph));
        }
        else {
            result.push(...propertyObj[namespaces_1._RDFS.subPropertyOf]);
        }
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    getSubProperties(implicit = true, filter) {
        const propertyObj = this.getTermObj();
        const result = [];
        if (implicit) {
            result.push(...(0, reasoning_1.inferSubProperties)(this.IRI, this.graph));
        }
        else {
            result.push(...propertyObj[namespaces_1._SOA.superPropertyOf]);
        }
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    getInverseOf() {
        const propertyObj = this.getTermObj();
        return propertyObj[namespaces_1._SCHEMA.inverseOf];
    }
    toString(implicit = true, filter) {
        return JSON.stringify(this.toJSON(implicit, filter), null, 2);
    }
    toJSON(implicit = true, filter) {
        const result = super.toJSON();
        result["ranges"] = this.getRanges(implicit, filter);
        result["domains"] = this.getDomains(implicit, filter);
        result["superProperties"] = this.getSuperProperties(implicit, filter);
        result["subProperties"] = this.getSubProperties(implicit, filter);
        result["inverseOf"] = this.getInverseOf();
        return result;
    }
}
exports.Property = Property;
//# sourceMappingURL=Property.js.map