"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Term = void 0;
const utilities_1 = require("./utilities");
const namespaces_1 = require("./namespaces");
class Term {
    constructor(IRI, graph) {
        this.IRI = IRI;
        this.graph = graph;
    }
    getIRI(compactForm = false) {
        if (compactForm) {
            return this.IRI;
        }
        return (0, utilities_1.toAbsoluteIRI)(this.IRI, this.graph.context);
    }
    getTermTypeLabel() {
        return this.termTypeLabel;
    }
    getTermTypeIRI() {
        return this.termTypeIRI;
    }
    getVocabURLs() {
        const termObj = this.getTermObj();
        if (!(0, utilities_1.isNil)(termObj["vocabURLs"])) {
            return termObj["vocabURLs"];
        }
        return null;
    }
    getVocabulary() {
        const termObj = this.getTermObj();
        if (!(0, utilities_1.isNil)(termObj[namespaces_1._SCHEMA.isPartOf])) {
            return termObj[namespaces_1._SCHEMA.isPartOf];
        }
        return null;
    }
    getSource() {
        const termObj = this.getTermObj();
        if (!(0, utilities_1.isNil)(termObj[namespaces_1._DC.source])) {
            return termObj[namespaces_1._DC.source];
        }
        else if (!(0, utilities_1.isNil)(termObj[namespaces_1._SCHEMA.source])) {
            return termObj[namespaces_1._SCHEMA.source];
        }
        return null;
    }
    isSupersededBy() {
        const termObj = this.getTermObj();
        if ((0, utilities_1.isString)(termObj[namespaces_1._SCHEMA.supersededBy])) {
            return termObj[namespaces_1._SCHEMA.supersededBy];
        }
        return null;
    }
    getName(language = "en") {
        const termObj = this.getTermObj()[namespaces_1._RDFS.label];
        if ((0, utilities_1.isNil)(termObj) || (0, utilities_1.isNil)(termObj[language])) {
            return null;
        }
        return termObj[language];
    }
    getDescription(language = "en") {
        const termObj = this.getTermObj()[namespaces_1._RDFS.comment];
        if ((0, utilities_1.isNil)(termObj) || (0, utilities_1.isNil)(termObj[language])) {
            return null;
        }
        return termObj[language];
    }
    toString() {
        return JSON.stringify(this.toJSON(), null, 2);
    }
    toJSON() {
        return {
            id: this.getIRI(true),
            IRI: this.getIRI(),
            typeLabel: this.getTermTypeLabel(),
            typeIRI: this.getTermTypeIRI(),
            vocabURLs: this.getVocabURLs(),
            vocabulary: this.getVocabulary(),
            source: this.getSource(),
            supersededBy: this.isSupersededBy(),
            name: this.getName(),
            description: this.getDescription(),
        };
    }
}
exports.Term = Term;
//# sourceMappingURL=Term.js.map