"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.inferRangeOf = exports.inferSubProperties = exports.inferSuperProperties = exports.inferSubDataTypes = exports.inferSuperDataTypes = exports.inferSubClasses = exports.inferSuperClasses = exports.inferPropertiesFromSuperClasses = exports.applyFilter = void 0;
const utilities_1 = require("./utilities");
const namespaces_1 = require("./namespaces");
function applyFilter(paramObj) {
    const { data, filter, graph } = paramObj;
    if (!Array.isArray(data) ||
        data.length === 0 ||
        !filter ||
        Object.keys(filter).length === 0) {
        return data;
    }
    const unifiedDataArray = (0, utilities_1.uniquifyArray)(data);
    const result = [];
    const context = graph.context;
    let namespaces;
    if (filter.fromVocabulary) {
        namespaces = (0, utilities_1.toArray)(filter.fromVocabulary);
        for (let v = 0; v < namespaces.length; v++) {
            for (let vi = 0; vi < Object.keys(context).length; vi++) {
                if (context[Object.keys(context)[vi]] === namespaces[v]) {
                    namespaces[v] = Object.keys(context)[vi];
                    break;
                }
            }
        }
    }
    for (let i = 0; i < unifiedDataArray.length; i++) {
        const actualTerm = graph.getTerm(unifiedDataArray[i]);
        if (!actualTerm) {
            continue;
        }
        if (filter.isSuperseded !== undefined) {
            if (!filter.isSuperseded && actualTerm.isSupersededBy() != null) {
                continue;
            }
            else if (filter.isSuperseded && actualTerm.isSupersededBy() == null) {
                continue;
            }
        }
        if (namespaces) {
            let matchFound = false;
            for (let v = 0; v < namespaces.length; v++) {
                if (actualTerm.getIRI(true).startsWith(namespaces[v])) {
                    matchFound = true;
                    break;
                }
            }
            if (!matchFound) {
                continue;
            }
        }
        if (filter.termType) {
            const toCheck = (0, utilities_1.toArray)(filter.termType);
            const invalidTermType = toCheck.find((el) => !Object.values(namespaces_1.TermTypeLabel).includes(el));
            if (invalidTermType) {
                throw new Error("Invalid filter.termType " + invalidTermType);
            }
            const foundMatch = toCheck.find((el) => el === actualTerm.getTermTypeLabel());
            if (!foundMatch) {
                continue;
            }
        }
        result.push(unifiedDataArray[i]);
    }
    return result;
}
exports.applyFilter = applyFilter;
function inferPropertiesFromSuperClasses(superClasses, graph) {
    const result = [];
    for (const superClass of superClasses) {
        const superClassObj = graph.classes[superClass] || graph.enumerations[superClass];
        if (superClassObj) {
            result.push(...superClassObj[namespaces_1._SOA.hasProperty]);
            if (superClassObj[namespaces_1._RDFS.subClassOf].length !== 0) {
                result.push(...inferPropertiesFromSuperClasses(superClassObj[namespaces_1._RDFS.subClassOf], graph));
            }
        }
    }
    return (0, utilities_1.uniquifyArray)(result);
}
exports.inferPropertiesFromSuperClasses = inferPropertiesFromSuperClasses;
function inferSuperClasses(classIRI, graph) {
    let result = [];
    const classObj = graph.classes[classIRI] || graph.enumerations[classIRI];
    if (classObj) {
        result.push(...classObj[namespaces_1._RDFS.subClassOf]);
        let addition = (0, utilities_1.cloneJson)(result);
        do {
            let newAddition = [];
            for (const curAdd of addition) {
                const parentClassObj = graph.classes[curAdd] || graph.enumerations[curAdd];
                if (parentClassObj) {
                    newAddition.push(...parentClassObj[namespaces_1._RDFS.subClassOf]);
                }
            }
            newAddition = (0, utilities_1.uniquifyArray)(newAddition);
            addition = (0, utilities_1.cloneJson)(newAddition);
            result.push(...newAddition);
        } while (addition.length !== 0);
        result = (0, utilities_1.uniquifyArray)(result);
    }
    return result;
}
exports.inferSuperClasses = inferSuperClasses;
function inferSubClasses(classIRI, graph) {
    let result = [];
    const classObj = graph.classes[classIRI] || graph.enumerations[classIRI];
    if (classObj) {
        result.push(...classObj[namespaces_1._SOA.superClassOf]);
        let addition = (0, utilities_1.cloneJson)(result);
        do {
            let newAddition = [];
            for (const curAdd of addition) {
                const parentClassObj = graph.classes[curAdd] || graph.enumerations[curAdd];
                if (parentClassObj) {
                    newAddition.push(...parentClassObj[namespaces_1._SOA.superClassOf]);
                }
            }
            newAddition = (0, utilities_1.uniquifyArray)(newAddition);
            addition = (0, utilities_1.cloneJson)(newAddition);
            result.push(...newAddition);
        } while (addition.length !== 0);
        result = (0, utilities_1.uniquifyArray)(result);
    }
    return result;
}
exports.inferSubClasses = inferSubClasses;
function inferSuperDataTypes(dataTypeIRI, graph) {
    let result = [];
    const dataTypeObj = graph.dataTypes[dataTypeIRI];
    if (dataTypeObj) {
        result.push(...dataTypeObj[namespaces_1._RDFS.subClassOf]);
        let addition = (0, utilities_1.cloneJson)(result);
        do {
            let newAddition = [];
            for (const curAdd of addition) {
                const parentDataTypeObj = graph.dataTypes[curAdd];
                if (parentDataTypeObj) {
                    newAddition.push(...parentDataTypeObj[namespaces_1._RDFS.subClassOf]);
                }
            }
            newAddition = (0, utilities_1.uniquifyArray)(newAddition);
            addition = (0, utilities_1.cloneJson)(newAddition);
            result.push(...newAddition);
        } while (addition.length !== 0);
        result = (0, utilities_1.uniquifyArray)(result);
    }
    return result;
}
exports.inferSuperDataTypes = inferSuperDataTypes;
function inferSubDataTypes(dataTypeIRI, graph) {
    let result = [];
    const dataTypeObj = graph.dataTypes[dataTypeIRI];
    if (dataTypeObj) {
        result.push(...dataTypeObj[namespaces_1._SOA.superClassOf]);
        let addition = (0, utilities_1.cloneJson)(result);
        do {
            let newAddition = [];
            for (const curAdd of addition) {
                const childDataTypeObj = graph.dataTypes[curAdd];
                if (childDataTypeObj) {
                    newAddition.push(...childDataTypeObj[namespaces_1._SOA.superClassOf]);
                }
            }
            newAddition = (0, utilities_1.uniquifyArray)(newAddition);
            addition = (0, utilities_1.cloneJson)(newAddition);
            result.push(...newAddition);
        } while (addition.length !== 0);
        result = (0, utilities_1.uniquifyArray)(result);
    }
    return result;
}
exports.inferSubDataTypes = inferSubDataTypes;
function inferSuperProperties(propertyIRI, graph) {
    let result = [];
    const propertyObj = graph.properties[propertyIRI];
    if (propertyObj) {
        result.push(...propertyObj[namespaces_1._RDFS.subPropertyOf]);
        let addition = (0, utilities_1.cloneJson)(result);
        do {
            let newAddition = [];
            for (const curAdd of addition) {
                const parentPropertyObj = graph.properties[curAdd];
                if (parentPropertyObj) {
                    newAddition.push(...parentPropertyObj[namespaces_1._RDFS.subPropertyOf]);
                }
            }
            newAddition = (0, utilities_1.uniquifyArray)(newAddition);
            addition = (0, utilities_1.cloneJson)(newAddition);
            result.push(...newAddition);
        } while (addition.length !== 0);
        result = (0, utilities_1.uniquifyArray)(result);
    }
    return result;
}
exports.inferSuperProperties = inferSuperProperties;
function inferSubProperties(propertyIRI, graph) {
    let result = [];
    const propertyObj = graph.properties[propertyIRI];
    if (propertyObj) {
        result.push(...propertyObj[namespaces_1._SOA.superPropertyOf]);
        let addition = (0, utilities_1.cloneJson)(result);
        do {
            let newAddition = [];
            for (const curAdd of addition) {
                const parentPropertyObj = graph.properties[curAdd];
                if (parentPropertyObj) {
                    newAddition.push(...parentPropertyObj[namespaces_1._SOA.superPropertyOf]);
                }
            }
            newAddition = (0, utilities_1.uniquifyArray)(newAddition);
            addition = (0, utilities_1.cloneJson)(newAddition);
            result.push(...newAddition);
        } while (addition.length !== 0);
        result = (0, utilities_1.uniquifyArray)(result);
    }
    return result;
}
exports.inferSubProperties = inferSubProperties;
function inferRangeOf(rangeIRI, graph) {
    const classObj = graph.classes[rangeIRI] || graph.enumerations[rangeIRI];
    const result = [];
    if (classObj) {
        result.push(...classObj[namespaces_1._SOA.isRangeOf]);
        const superClasses = inferSuperClasses(rangeIRI, graph);
        for (const superClass of superClasses) {
            const superClassObj = graph.classes[superClass] || graph.enumerations[superClass];
            if (superClassObj) {
                result.push(...superClassObj[namespaces_1._SOA.isRangeOf]);
            }
        }
    }
    else {
        const dataTypeObj = graph.dataTypes[rangeIRI];
        if (dataTypeObj) {
            result.push(...dataTypeObj[namespaces_1._SOA.isRangeOf]);
            const superDataTypes = inferSuperDataTypes(rangeIRI, graph);
            for (const superDataType of superDataTypes) {
                const superDataTypeObj = graph.dataTypes[superDataType];
                if (superDataTypeObj) {
                    result.push(...superDataTypeObj[namespaces_1._SOA.isRangeOf]);
                }
            }
        }
    }
    return (0, utilities_1.uniquifyArray)(result);
}
exports.inferRangeOf = inferRangeOf;
//# sourceMappingURL=reasoning.js.map