"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLatestSchemaVersion = exports.checkURL = exports.fetchSchemaVersions = exports.constructURLSchemaVocabulary = exports.getURLSchemaVersions = exports.getURLBaseSchema = exports.toArray = exports.switchIRIProtocol = exports.getFileNameForSchemaOrgVersion = exports.sortReleaseEntriesByDate = exports.toAbsoluteIRI = exports.toCompactIRI = exports.uniquifyArray = exports.isArray = exports.isString = exports.isNil = exports.isLanguageObjectVocab = exports.isObject = exports.cloneJson = void 0;
const axios_1 = __importDefault(require("axios"));
const RetrievalMemory_1 = require("./RetrievalMemory");
const myRetrievalMemory = RetrievalMemory_1.RetrievalMemory.getInstance();
function cloneJson(input) {
    if (input === undefined) {
        return input;
    }
    return JSON.parse(JSON.stringify(input));
}
exports.cloneJson = cloneJson;
function isObject(value) {
    if (Array.isArray(value)) {
        return false;
    }
    if (isNil(value)) {
        return false;
    }
    return typeof value === "object";
}
exports.isObject = isObject;
function isLanguageObjectVocab(value) {
    if (isObject(value)) {
        if (isString(value["@language"]) && isString(value["@value"])) {
            return true;
        }
    }
    return false;
}
exports.isLanguageObjectVocab = isLanguageObjectVocab;
function isNil(value) {
    return value === undefined || value === null;
}
exports.isNil = isNil;
function isString(value) {
    if (isNil(value)) {
        return false;
    }
    return typeof value === "string" || value instanceof String;
}
exports.isString = isString;
function isArray(value) {
    return Array.isArray(value);
}
exports.isArray = isArray;
function uniquifyArray(array) {
    return [...new Set(array)];
}
exports.uniquifyArray = uniquifyArray;
function toCompactIRI(absoluteIRI, context, equateVocabularyProtocols = false) {
    for (const contextTerm of Object.keys(context)) {
        const vocabIRI = context[contextTerm];
        if (isString(vocabIRI) && absoluteIRI.startsWith(vocabIRI)) {
            return (contextTerm + ":" + absoluteIRI.substring(vocabIRI.length));
        }
        if (equateVocabularyProtocols && isString(vocabIRI)) {
            const protocolSwitchedIRI = switchIRIProtocol(vocabIRI);
            if (absoluteIRI.startsWith(protocolSwitchedIRI)) {
                return (contextTerm + ":" + absoluteIRI.substring(protocolSwitchedIRI.length));
            }
        }
    }
    throw new Error("Trying to get a compact IRI for a term with no entry in the Context");
}
exports.toCompactIRI = toCompactIRI;
function toAbsoluteIRI(compactIRI, context) {
    const terms = Object.keys(context);
    for (let i = 0; i < terms.length; i++) {
        const vocabIRI = context[terms[i]];
        if (compactIRI.substring(0, compactIRI.indexOf(":")) === terms[i]) {
            return vocabIRI.concat(compactIRI.substring(compactIRI.indexOf(":") + 1));
        }
    }
    throw new Error("Trying to get an absolute IRI for a term with no entry in the Context");
}
exports.toAbsoluteIRI = toAbsoluteIRI;
function sortReleaseEntriesByDate(releaseLog) {
    const versionEntries = Object.entries(releaseLog);
    return versionEntries.sort((a, b) => +new Date(b[1]) - +new Date(a[1]));
}
exports.sortReleaseEntriesByDate = sortReleaseEntriesByDate;
function getFileNameForSchemaOrgVersion(version, schemaHttps = true) {
    switch (version) {
        case "2.0":
        case "2.1":
        case "2.2":
        case "3.0":
            throw new Error("There is no jsonld file for that schema.org version.");
        case "3.1":
        case "3.2":
        case "3.3":
        case "3.4":
        case "3.5":
        case "3.6":
        case "3.7":
        case "3.8":
        case "3.9":
        case "4.0":
        case "5.0":
        case "6.0":
        case "7.0":
        case "7.01":
        case "7.02":
        case "7.03":
        case "7.04":
        case "8.0":
            return "all-layers.jsonld";
        case "9.0":
            if (schemaHttps) {
                return "schemaorg-all-https.jsonld";
            }
            else {
                return "schemaorg-all-http.jsonld";
            }
        default:
            if (schemaHttps) {
                return "schemaorg-all-https.jsonld";
            }
            else {
                return "schemaorg-all-http.jsonld";
            }
    }
}
exports.getFileNameForSchemaOrgVersion = getFileNameForSchemaOrgVersion;
function switchIRIProtocol(IRI) {
    if (IRI.startsWith("https://")) {
        return "http" + IRI.substring(5);
    }
    else if (IRI.startsWith("http://")) {
        return "https" + IRI.substring(4);
    }
    return IRI;
}
exports.switchIRIProtocol = switchIRIProtocol;
const toArray = (o) => (Array.isArray(o) ? o : [o]);
exports.toArray = toArray;
const URI_SEMANTIFY_GITHUB = "https://raw.githubusercontent.com/semantifyit/schemaorg/main";
const URI_SCHEMA_ORG_GITHUB = "https://raw.githubusercontent.com/schemaorg/schemaorg/";
function getURLBaseSchema(commitBase) {
    const path = "/data/releases/";
    if (commitBase) {
        return URI_SCHEMA_ORG_GITHUB + commitBase + path;
    }
    else {
        return URI_SEMANTIFY_GITHUB + path;
    }
}
exports.getURLBaseSchema = getURLBaseSchema;
function getURLSchemaVersions(commitBase) {
    const path = "/versions.json";
    if (commitBase) {
        return URI_SCHEMA_ORG_GITHUB + commitBase + path;
    }
    else {
        return URI_SEMANTIFY_GITHUB + path;
    }
}
exports.getURLSchemaVersions = getURLSchemaVersions;
async function constructURLSchemaVocabulary(version = "latest", schemaHttps = true, commit) {
    if (version === "latest") {
        version = await getLatestSchemaVersion(commit);
    }
    const fileName = getFileNameForSchemaOrgVersion(version, schemaHttps);
    return getURLBaseSchema(commit) + version + "/" + fileName;
}
exports.constructURLSchemaVocabulary = constructURLSchemaVocabulary;
async function fetchSchemaVersions(cacheClear = false, commit) {
    let versionFile;
    if (cacheClear) {
        myRetrievalMemory.deleteCache();
    }
    else {
        const cachedData = myRetrievalMemory.getData("versionsFile", commit);
        if (cachedData) {
            return cachedData;
        }
    }
    const urlSchemaVersions = getURLSchemaVersions(commit);
    try {
        versionFile = await axios_1.default.get(urlSchemaVersions);
    }
    catch (e) {
        throw new Error("Unable to retrieve the schema.org versions file at " + urlSchemaVersions);
    }
    if (!versionFile || !versionFile.data || !versionFile.data.releaseLog) {
        throw new Error("The schema.org versions file at " +
            urlSchemaVersions +
            " returned an unexpected result.");
    }
    const schemaVersions = versionFile.data;
    myRetrievalMemory.setData("versionsFile", schemaVersions, commit);
    let latestVersion;
    if (schemaVersions.schemaversion &&
        (await checkURL(await constructURLSchemaVocabulary(schemaVersions.schemaversion)))) {
        latestVersion = schemaVersions.schemaversion;
    }
    else {
        const sortedArray = sortReleaseEntriesByDate(schemaVersions.releaseLog);
        for (const currVersion of sortedArray) {
            if (await checkURL(await constructURLSchemaVocabulary(currVersion[0]))) {
                latestVersion = currVersion[0];
                break;
            }
        }
    }
    if (!latestVersion) {
        throw new Error('Could not find any valid vocabulary file in the schema.org versions to be declared as "latest".');
    }
    myRetrievalMemory.setData("latest", latestVersion, commit);
    return schemaVersions;
}
exports.fetchSchemaVersions = fetchSchemaVersions;
async function checkURL(url) {
    try {
        await axios_1.default.head(url);
        return true;
    }
    catch (e) {
        return false;
    }
}
exports.checkURL = checkURL;
async function getLatestSchemaVersion(commit) {
    let latestVersion = myRetrievalMemory.getData("latest", commit);
    if (!latestVersion) {
        await fetchSchemaVersions(false, commit);
    }
    latestVersion = myRetrievalMemory.getData("latest", commit);
    if (latestVersion) {
        return latestVersion;
    }
    else {
        throw new Error("Could not identify the latest version of the schema.org vocabulary");
    }
}
exports.getLatestSchemaVersion = getLatestSchemaVersion;
//# sourceMappingURL=utilities.js.map